"""Tagulous TagField support."""

from dal.fields import CreateModelMultipleField


class TagulousField(CreateModelMultipleField):
    """FutureModelForm support for tagulous TagField."""

    def value_from_object(self, instance, name):
        """Return a list of Tags for the TagField."""
        return None if not instance.pk else [
            x for x in getattr(instance, name).all()]



"""Select2 view implementation."""

import json

from dal.views import BaseQuerySetView

from django import http


class Select2ViewMixin(object):
    """View mixin to render a JSON response for Select2."""

    def get_results(self, context):
        """Return data for the 'results' key of the response."""
        return [
            {
                'id': self.get_result_value(result),
                'text': self.get_result_label(result),
            } for result in context['object_list']
        ]

    def render_to_response(self, context):
        """Return a JSON response in Select2 format."""
        create_option = []

        q = self.request.GET.get('q', None)
        if self.request.GET.get('create', None) == 'true' and q:
            create_option = [{
                'id': q,
                'text': 'Create "%s"' % q,
            }]

        return http.HttpResponse(
            json.dumps({
                'results': self.get_results(context) + create_option,
                'more': self.has_more(context)
            }),
            content_type='application/json',
        )


class Select2QuerySetView(Select2ViewMixin, BaseQuerySetView):
    """List options for a Select2 widget."""



"""Autocomplete form fields for django-taggit."""

from dal.fields import CreateModelMultipleField

from taggit.models import Tag


class TaggitField(CreateModelMultipleField):
    """Form field for django-taggit."""

    def __init__(self, queryset=None, *args, **kwargs):
        """Make Tag.objects.all() the default queryset."""
        kwargs['queryset'] = queryset or Tag.objects.all()
        super(TaggitField, self).__init__(*args, **kwargs)

    def value_from_object(self, instance, name):
        """Retrieve the initial form value."""
        return None if not instance.pk else [
            x for x in getattr(instance, name).all()]



"""GM2MField support for autocomplete fields."""


class GM2MFieldMixin(object):
    """GM2MField ror FutureModelForm."""

    def value_from_object(self, instance, name):
        """Return the list of objects in the GM2MField relation."""
        return None if not instance.pk else [
            x for x in getattr(instance, name).all()]

    def save_relation_data(self, instance, name, value):
        """Save the relation into the GM2MField."""
        setattr(instance, name, value)



"""Utils for testing autocompletes."""
from django.apps import apps


class Fixtures(object):
    """Callback for post_migrate to create many objects."""

    def __init__(self, model_name=None):
        """Preset a model name, ie. 'auth.user'."""
        self.model_name = model_name

    def get_model(self, sender):
        """Return either the preset model, either the sender's TestModel."""
        if self.model_name is None:
            return sender.get_model('TestModel')
        else:
            return apps.get_model(self.model_name)

    def __call__(self, sender, **kwargs):
        """Callback function, calls install_fixtures."""
        model = self.get_model(sender)
        self.install_fixtures(model)

    def install_fixtures(self, model):
        """Install fixtures for model."""
        for n in range(1, 50):
            try:
                model.objects.get(pk=n)
            except model.DoesNotExist:
                model.objects.create(name='test %s' % n, pk=n)


class OwnedFixtures(Fixtures):
    """Fixtures for models with an "owner" relation to User."""

    installed_auth = False

    def install_fixtures(self, model):
        """Install owners and fixtures."""
        if not self.installed_auth:
            User = apps.get_model('auth.user')  # noqa

            self.test, c = User.objects.get_or_create(
                username='test',
                is_staff=True,
                is_superuser=True
            )
            self.test.set_password('test')
            self.test.save()

            self.other, c = User.objects.get_or_create(username='other')
            self.other.set_password('test')
            self.other.save()

            self.installed_auth = True

        for n in range(1, 3):
            for u in [self.test, self.other]:
                model.objects.get_or_create(
                    name='test #%s for %s' % (n, u),
                    owner=u
                )

fixtures = Fixtures()



"""Form fields which may create missing models."""

from django import forms


class CreateModelFieldMixin(object):
    """Mixin for autocomplete form fields with create power."""

    def create_value(self, value):
        """Create and return a model from a user value."""
        return self.queryset.model.objects.create(name=value).pk

    def widget_attrs(self, widget):
        """Override to data-autocomplete-light-create to 'true'."""
        attrs = super(CreateModelFieldMixin, self).widget_attrs(widget)
        attrs['data-autocomplete-light-create'] = 'true'
        return attrs


class CreateModelField(CreateModelFieldMixin, forms.ModelChoiceField):
    """This field allows creating instances."""

    def clean(self, value):
        """Try the default clean method, else create if allowed."""
        try:
            return super(CreateModelFieldMixin, self).clean(value)
        except forms.ValidationError:
            if value:
                value = self.create_value(value)
            else:
                raise

        return super(CreateModelFieldMixin, self).clean(value)


class CreateModelMultipleField(CreateModelFieldMixin,
                               forms.ModelMultipleChoiceField):
    """This field allows creating instances."""

    def clean(self, value):
        """Try the default clean method, else create if allowed."""
        tries = len(value)

        while tries >= 0:
            try:
                return super(CreateModelMultipleField, self).clean(value)
            except forms.ValidationError as e:
                if e.params.get('pk', None):
                    new_value = self.create_value(e.params['pk'])
                    value[value.index(e.params['pk'])] = new_value
                else:
                    raise
            tries -= 1

        return super(CreateModelMultipleField, self).clean(value)



"""
Django-autocomplete-light extension for django-querysetsequence.

It provides view encapsulating business-logic for QuerySetSequence in
:py:class:`~dal_queryset_sequence.views.BaseQuerySetSequenceView` as well as
the widget encapsulating business logic for rendering only selected options in
:py:class:~dal_queryset_sequence.widgets.QuerySetSequenceSelectMixin`.

Add `dal_queryset_sequence` to INSTALLED_APPS to enable it in
:py:mod:`dal.autocomplete` shortcut module.

For example usage, refer to the ``select2_generic_foreign_key`` example app in
the ``test_project``.
"""
